#include "myglwidget.h"

MyGLWidget::MyGLWidget(QWidget *parent)
    : QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
{
    tesselation = 1;
    cameraZoom = -5;
    translationX = 0;
    translationY = 0;
    clickStartPosition = QPoint(0,0);
     rightButtonPressed = false;
}

MyGLWidget::~MyGLWidget()
{
    /*http://doc.qt.io/qt-4.8/qt-opengl-hellogl-example.html :
    * We use a destructor to ensure that any OpenGL-specific
    * data structures are deleted when the widget is no longer
    * needed (although in this case nothing needs cleaning up).
    */
    //In helloGL example: nothin needs cleaning up.
}

/*
 * We provide size hint functions to ensure that
 * the widgetis shown at a reasonable size:
 */
QSize MyGLWidget::minimumSizeHint() const
{
    return QSize(50, 50);
}

/*
 * We provide size hint functions to ensure that
 * the widgetis shown at a reasonable size:
 */
QSize MyGLWidget::sizeHint() const
{
    return QSize(600, 400);
}


void MyGLWidget::initializeGL()
{
    glClearColor(0.0, 0.0, 0.0, 0.0);//TODO brauch ich das?
    glEnable(GL_DEPTH_TEST);//TODO warum genau?

//    glEnable(GL_CULL_FACE);//backface culling
//    glCullFace(GL_BACK);//backface culling

//    glEnable(GL_COLOR_MATERIAL); //Habe ich für glColor3f gebraucht. Wieder raus für Material. //In order to use both color and lighting you must activate openGL color materials

    //4.1.1
    /*
     * The initial state of the OpenGL state machine is defined in
initializeGL(). Furthermore, you can set all OpenGL parameters
that will not change during program execution here as well, for
example, the background color.
     */

    glEnable(GL_LIGHTING);//enable lighting.
    glEnable(GL_LIGHT0);//enable an OpenGL light
 /*
  * https://msdn.microsoft.com/en-us/library/windows/desktop/dd373578%28v=vs.85%29.aspx
  * The glLightfv function returns light source parameter values.
  * GL_POSITION:
  * The params parameter contains four floating-point values
  * that specify the position of the light in homogeneous object
  * coordinates. Both integer and floating-point values are
  * mapped directly. Neither integer nor floating-point values
  * are clamped.
 */
    static GLfloat lightPosition[4] = {0.5, 0.0, 2.0, 1.0};
    glLightfv(GL_LIGHT0,GL_POSITION,lightPosition);//set Position of light source.


    //HelloGL Example:
/*
    qglClearColor(qtPurple.dark());

    logo = new QtLogo(this, 64);
    logo->setColor(qtGreen.dark());

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glShadeModel(GL_SMOOTH);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_MULTISAMPLE);
    static GLfloat lightPosition[4] = { 0.5, 5.0, 7.0, 1.0 };
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
*/
}

void MyGLWidget::paintGL(){

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

    glTranslatef(translationX,translationY,cameraZoom);//Kamera sitzt sonst in (0,0,0) und man sieht nichts (duh)

    double qubeWidth = 1.0;
    double dist = qubeWidth/(double)tesselation;

    //4.1.1 Unit Qube + Tesselation + Diffuse Material
    {
    glBegin(GL_QUADS);

    //TODO später einfügen.
//    GLfloat specularColor[] = {1,1,1};
//    GLfloat shininess[] = {100};//specular exponent
//    glMaterialfv(GL_FRONT,GL_SPECULAR,specularColor);
//    glMaterialfv(GL_FRONT,GL_SHININESS,shininess);

    //vorne // z-Achse
//    glColor3f(0,0,1);//blau
    GLfloat blue[] = {0,0,1};
    glMaterialfv(GL_FRONT,GL_DIFFUSE,blue);
    for (int i = 0; i < tesselation; i++){
        for (int j = 0; j < tesselation; j++){
            glVertex3f(0.5-dist*i,0.5-dist*j,0.5);
            glVertex3f(0.5-dist*(i+1),0.5-dist*j,0.5);
            glVertex3f(0.5-dist*(i+1),0.5-dist*(j+1),0.5);
            glVertex3f(0.5-dist*i,0.5-dist*(j+1),0.5);
        }
    }
//    glVertex3f(0.5, 0.5, 0.5);//4
//    glVertex3f(-0.5, 0.5, 0.5);//3
//    glVertex3f(-0.5, -0.5, 0.5);//2
//    glVertex3f(0.5, -0.5, 0.5);//1

    //oben // y-Achse
    GLfloat green[] = {0,1,0};
    glMaterialfv(GL_FRONT,GL_DIFFUSE,green);
    for (int i = 0; i < tesselation; i++){
        for (int j = 0; j < tesselation; j++){
            glVertex3f(0.5-dist*(i+1), 0.5, 0.5-dist*(j+1));
            glVertex3f(0.5-dist*(i+1), 0.5, 0.5-dist*j);
            glVertex3f(0.5-dist*i, 0.5, 0.5-dist*j);
            glVertex3f(0.5-dist*i, 0.5, 0.5-dist*(j+1));
        }
    }


    //rechts //x-Achse
    GLfloat red[] = {1,0,0};
    glMaterialfv(GL_FRONT,GL_DIFFUSE,red);
    for (int i = 0; i < tesselation; i++){
        for (int j = 0; j < tesselation; j++){
            glVertex3f(0.5, 0.5-dist*i, 0.5-dist*j);
            glVertex3f(0.5, 0.5-dist*(i+1), 0.5-dist*j);
            glVertex3f(0.5, 0.5-dist*(i+1), 0.5-dist*(j+1));
            glVertex3f(0.5, 0.5-dist*i, 0.5-dist*(j+1));
        }
    }


    //hinten // z-Achse
    GLfloat yellow[] = {1,0.9,0};
    glMaterialfv(GL_FRONT,GL_DIFFUSE,yellow);
    for (int i = 0; i < tesselation; i++){
        for (int j = 0; j < tesselation; j++){
            glVertex3f(0.5-dist*i, 0.5-dist*(j+1), -0.5);//4
            glVertex3f(0.5-dist*(i+1), 0.5-dist*(j+1), -0.5);//3
            glVertex3f(0.5-dist*(i+1), 0.5-dist*j, -0.5);//2
            glVertex3f(0.5-dist*i, 0.5-dist*j, -0.5);//1
        }
    }

    //links
    GLfloat cyan[] = {0,1,1};
    glMaterialfv(GL_FRONT,GL_DIFFUSE,cyan);
    for (int i = 0; i < tesselation; i++){
        for (int j = 0; j < tesselation; j++){
            glVertex3f(-0.5, 0.5-dist*i, 0.5-dist*(j+1));
            glVertex3f(-0.5, 0.5-dist*(i+1), 0.5-dist*(j+1));
            glVertex3f(-0.5, 0.5-dist*(i+1), 0.5-dist*j);
            glVertex3f(-0.5, 0.5-dist*i, 0.5-dist*j);
        }
    }

    //unten //-y
    GLfloat magenta[] = {1,0,1};
    glMaterialfv(GL_FRONT,GL_DIFFUSE,magenta);
    for (int i = 0; i < tesselation; i++){
        for (int j = 0; j < tesselation; j++){
            glVertex3f(0.5-dist*i, -0.5, 0.5-dist*(j+1));
            glVertex3f(0.5-dist*i, -0.5, 0.5-dist*j);
            glVertex3f(0.5-dist*(i+1), -0.5, 0.5-dist*j);
            glVertex3f(0.5-dist*(i+1), -0.5, 0.5-dist*(j+1));
        }
    }


    glEnd();
    }
}

void MyGLWidget::resizeGL(int width, int height){
    double p = (double)width / (double)height;
        qDebug() << "Resize to " << width << "," << height << ", Perspective: " << p;
//old:
//    GLInt x = 0;//The lower-left corner of the viewport rectangle, in pixels. The default is (0,0).
//    GLInt y = 0;//The lower-left corner of the viewport rectangle, in pixels. The default is (0,0).
//    glViewport(x,y,width,height);//The width of the viewport. When an OpenGL context is first attached to a window, width and height are set to the dimensions of that window.
    glViewport(0,0,width,height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(45.0,p,0.1,1000);//TODO komisch: wenn schmaler als hoch auf einmal grau.

    glMatrixMode(GL_MODELVIEW);

}

void MyGLWidget::mousePressEvent(QMouseEvent *event){
    rightButtonPressed = false;
    if (event->button() == Qt::RightButton){
        rightButtonPressed = true;
        clickStartPosition = event->pos();
        qDebug() << "mouse press at " << clickStartPosition;
    }
}

void MyGLWidget:: mouseMoveEvent(QMouseEvent *event){
    //This didn't work properly if (event->button() == Qt::RightButton){
    if (rightButtonPressed){
        QPoint currentPosition = event->pos();
        QPoint diff = currentPosition - clickStartPosition;
        clickStartPosition = currentPosition;
        translationX -= 0.01*diff.x();
        translationY += 0.01*diff.y();
        updateGL();
    }
    qDebug() << "mouse move button " << event->button();
}

//4.1.2 User Input: Zooming
void MyGLWidget::wheelEvent(QWheelEvent *event){
    cameraZoom += 0.01*event->delta();
    qDebug() << "Cam zoom delta " << event->delta() << " stored " << cameraZoom;
    updateGL();
}


//4.1.1 slots for shading modes

void MyGLWidget::shadeWireframeSlot(){
    qDebug() << "Wireframe selected";
    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);//Wireframe looks way cooler with both faces on
    updateGL();
}

void MyGLWidget::shadeFlatSlot(){
    qDebug() << "Flat selected";
    glShadeModel(GL_FLAT);
    glPolygonMode(GL_FRONT,GL_FILL);
    updateGL();
}

void MyGLWidget::shadeGouraudSlot(){
    qDebug() << "Gouraud selected";
    glShadeModel(GL_SMOOTH);
    glPolygonMode(GL_FRONT,GL_FILL);
    updateGL();
}

void MyGLWidget::shadePhongSlot(){
    qDebug() << "Phong selected";
    updateGL();
}

//4.1.1 tesselation slot

void MyGLWidget::setTessellation(int t){
    qDebug() << "Tesselation: " << t;
    tesselation = t;
    updateGL();
}
